package ratRace.viewFX;

import javafx.geometry.Point2D;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;

import ratRace.model.DrawingAdapter;

public class FXDrawingAdapter implements DrawingAdapter {

	private final GraphicsContext grCxt;
	private final TransformFX xform ;
	private final double width;
	private final double height;

	FXDrawingAdapter(  GraphicsContext grCxt, TransformFX xform, double width, double height ) {
		this.grCxt = grCxt ;
		this.xform = xform ;
		this.width = width ;
		this.height = height ;
		
		grCxt.clearRect(0, 0, width, height);
		grCxt.setFill(Color.WHITE);
		grCxt.setStroke(Color.BLACK);
		grCxt.fillRect(0, 0, width, height);
		grCxt.strokeRect(0, 0, width, height);
	}
	
	@Override
	public void putMessage(String message, boolean gameOver) {
		// Draw the elapsed time.
		// TODO Get fancy with placement and size.
		grCxt.setStroke( Color.LIGHTBLUE ) ;
		grCxt.setFill( Color.LIGHTBLUE ) ;
		if( gameOver ) {
			Font bigFont = new Font(50) ;
			grCxt.setFont( bigFont ) ;
			grCxt.fillText(message, 20, 100) ;
			grCxt.strokeText(message, 20, 100) ;
		} else {
			Font smallFont = new Font(20) ;
			grCxt.setFont( smallFont ) ;
			grCxt.fillText(message, 20, 100) ;
			grCxt.strokeText(message, 20, 100) ; }
	}

	@Override
	public void drawWall(double i0, double j0, double i1, double j1) {
		Point2D p0 = xform.worldToView(i0, j0) ;
		Point2D p1 = xform.worldToView(i1, j1) ;
		grCxt.setStroke(Color.BLACK);
		grCxt.beginPath();
		grCxt.moveTo(p0.getX(), p0.getY());
		grCxt.lineTo(p1.getX(), p1.getY());
		grCxt.closePath() ;
		grCxt.stroke() ;
	}

	@Override
	public void drawGoal(double i, double j) {
		Point2D p0 = xform.worldToView(i, j) ;
		Point2D p1 = xform.worldToView(i+1, j+1) ;
		double x = p0.getX(), y = p0.getY();
		double w = p1.getX()-x, h = p1.getY()-y ;
		grCxt.setFill( Color.BLUE ) ;
		grCxt.fillRect((int)x, (int)y, (int)w+1, (int)h+1) ;
	}

	@Override
	public void drawRat(double i, double j, double radius) {
		Point2D p0 = xform.worldToView(i-radius, j-radius) ;
		Point2D p1 = xform.worldToView(i+radius, j+radius) ;
		double x = p0.getX(), y = p0.getY();
		double w = p1.getX()-x, h = p1.getY()-y ;
		grCxt.setFill( Color.RED) ;
		grCxt.fillOval((int)x, (int)y, (int)w+1, (int)h+1) ;
	}

}
