package ratRace.viewFX;

import java.util.Observable;
import java.util.Observer;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ToolBar;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import ratRace.controller.Controller;
import ratRace.model.Model;

public class RatRaceFX extends Application  implements Observer {
	private Model model ;
	private ViewFX view ;
	private Controller controller ;
	private Button goButton = new Button( "Go" ) ;
	private Button pauseButton = new Button( "Pause" ) ;
	private Button restartButton = new Button( "Restart" ) ;
	private BorderPane root ;
	
	void rebuildMVC() {
		if( view != null ) {
			view.terminate(); }
		model = new Model(50, 50) ;
		model.addObserver( this ) ;
		view = new ViewFX( model ) ;
		view.setStyle("-fx-background-color: #ff9999") ;
		controller = new Controller(model, view) ;
		root.setCenter( view ) ;
		update(null, null) ;
	}
    
	@Override
	public void update(Observable o, Object arg) {
		goButton.setDisable( ! model.getPaused() ) ;
		pauseButton.setDisable( model.getPaused() ) ;
	}
    
    @Override
    public void start(Stage primaryStage) {
        primaryStage.setTitle("RatRaceFX") ;

        goButton.setOnAction(
        	actionEvent -> controller.go() );
        pauseButton.setOnAction(
            actionEvent -> controller.pause() );
        restartButton.setOnAction(
            actionEvent -> rebuildMVC() );
        
        ToolBar toolBar = new ToolBar() ;
        toolBar.getItems().add( goButton ) ;
        toolBar.getItems().add( pauseButton ) ;
        toolBar.getItems().add( restartButton ) ;

        root = new BorderPane();
        root.setTop( toolBar ); 
        root.setStyle("-fx-background-color: #000000");
        Scene primaryScene = new Scene(root, 600, 600, Color.rgb(0,0,0)) ;
        primaryStage.setScene(primaryScene);
        rebuildMVC() ;
        primaryStage.show();
    }
	
    public static void main(String[] args) {
        launch(args);
    }
}