package ratRace.viewFX;

import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javafx.animation.AnimationTimer;
import javafx.event.EventHandler ;
import javafx.geometry.Point2D;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import ratRace.model.DrawingAdapter;
import ratRace.model.Model;
import ratRace.mvcInterfaces.ViewI;
import ratRace.mvcInterfaces.ViewListenerI;

public class ViewFX extends Canvas
       implements Observer, ViewI {
	
	private final Model model ;

	private final int modelWidth ;
	private final int modelHeight ;
	private final Set<ViewListenerI> myListeners 
    	= new HashSet<ViewListenerI>() ;

	
	private final AnimationTimer timer
		= new AnimationTimer() {
				@Override
				public void handle(long now) { pulse() ; } } ;

	public ViewFX( Model model ) {
		this.setWidth( 500 ) ;
		this.setHeight( 500 ) ;
		this.model = model ;
		modelWidth = model.getMazeWidth();
		modelHeight = model.getMazeHeight();
		EventHandler<MouseEvent> mouseMoveListener
			= event -> {
				double x = event.getX() ;
				double y = event.getY();
				TransformFX xform = new TransformFX( getWidth(), getHeight(),
				         model.getMazeWidth(), model.getMazeWidth() ) ;
				Point2D p = xform.viewToWorld(x, y) ;
				for( ViewListenerI listener : myListeners ) {
					listener.mouseMovedTo(p.getX(), p.getY()) ;
				} } ;
		this.setOnMouseMoved( mouseMoveListener ) ;
		model.addObserver( this ) ;
		timer.start() ;
		this.paint() ;
	}
	
	@Override
	public void addListener(ViewListenerI listener) {
		myListeners.add( listener ) ;
	}

	@Override
	public void update(Observable o, Object arg) {
		paint() ;
	}
	
	public void terminate() {
		timer.stop() ;
	}
	
	private void pulse() {
		for( ViewListenerI listener : myListeners ) {
			listener.pulse() ; }
	}
	
	private void paint() {
		GraphicsContext grCxt = this.getGraphicsContext2D() ;
		double width = getWidth() ;
		double height = getHeight() ;
		
		TransformFX xform = new TransformFX( width, height,
		         model.getMazeWidth(), model.getMazeWidth() ) ;
		
		DrawingAdapter da = new FXDrawingAdapter( grCxt, xform,
				width, height ) ;
		
		model.draw( da ); 
	}
}
